@@ -47,6 +47,9 @@ TMainF *MainF;
 #include <ZenLib/File.h>
 #include <ZenLib/FileName.h>
 #include <ZenLib/OS_Utils.h>
+
+#include <filesystem> // To remove when "Quick temporary fix for translations after an update" is removed
+
 using namespace MediaInfoNameSpace;
 using namespace ZenLib;
 //---------------------------------------------------------------------------
@@ -110,6 +113,52 @@ bool __fastcall TMainF::WindowsDarkModeEnabled()
     return DarkModeEnabled;
 }
 
+void __fastcall TMainF::ConfigTheme()
+{
+    switch(Prefs->Config(__T("Theme")).To_int32s()) {
+        case 0:
+            M_Options_Theme_System->Checked = true;
+            if (WindowsDarkModeEnabled()) {
+                TStyleManager::TrySetStyle(DARK_MODE_STYLE, false);
+            } else {
+                TStyleManager::TrySetStyle(LIGHT_MODE_STYLE, false);
+            }
+            break;
+        case 1:
+            M_Options_Theme_Light->Checked = true;
+            TStyleManager::TrySetStyle(LIGHT_MODE_STYLE, false);
+            break;
+        case 2:
+            M_Options_Theme_Dark->Checked = true;
+            TStyleManager::TrySetStyle(DARK_MODE_STYLE, false);
+            break;
+        default:
+            Prefs->Config(__T("Theme")) = __T("0");
+            M_Options_Theme_System->Checked = true;
+            if (WindowsDarkModeEnabled()) {
+                TStyleManager::TrySetStyle(DARK_MODE_STYLE, false);
+            } else {
+                TStyleManager::TrySetStyle(LIGHT_MODE_STYLE, false);
+            }
+            break;
+    }
+}
+
+//Function to inject a CSS style into html documents to make their look match the dark mode style
+std::wstring __fastcall TMainF::InjectDarkModeHTMLStyle(const wchar_t* HTMLDocument) {
+    const wchar_t* InsertionPoint = wcsstr(HTMLDocument, L"<head>");
+    const wchar_t* StyleContent = L"<style>body { background-color: #1F1F1F; color: #FFFFFF; }</style>";
+
+    if (InsertionPoint != nullptr) {
+        size_t InsertionPos = InsertionPoint - HTMLDocument + wcslen(L"<head>");
+        std::wstring ModifiedHTML(HTMLDocument);
+        ModifiedHTML.insert(InsertionPos, StyleContent);
+        return ModifiedHTML;
+    } else {
+        return HTMLDocument;
+    }
+}
+
 //---------------------------------------------------------------------------
 
 //***************************************************************************
@@ -199,12 +248,6 @@ __fastcall TMainF::TMainF(TComponent* Owner)
     Page_Position=-1;
     Caption=MEDIAINFO_TITLE;
 
-    //Set dark mode
-    if (WindowsDarkModeEnabled()) {
-        TStyleManager::TrySetStyle(DARK_MODE_STYLE, false);
-        M_Options_Darkmode->Checked=true;
-    }
-
     //Opt-out from styling dialogs and use native Windows dialogs for dark mode as well
     TStyleManager::SystemHooks = TStyleManager::SystemHooks -
                                  (TStyleManager::TSystemHooks() << TStyleManager::TSystemHook::shDialogs);
@@ -218,6 +261,32 @@ __fastcall TMainF::TMainF(TComponent* Owner)
         Page_Custom_Text->Font = monoFont;
         Page_Sheet_Text->Font = monoFont;
     }
+
+    //Configuration of MediaInfoLib
+    if (I == NULL)
+        I = new MediaInfoList;
+
+    //Load GUI preferences
+    GUI_Configure();
+
+    //File(s) in command line
+    #ifdef UNICODE
+        if (IsWin9X())
+        {
+            for (int I1 = 1; I1 <= ParamCount(); I1++)
+                I->Open(ParamStr(I1).c_str());
+        }
+        else
+        {
+            int argc;
+            MediaInfoNameSpace::Char** argv = CommandLineToArgvW(GetCommandLineW(), &argc);
+            for (int I1 = 1; I1 < argc; I1++)
+                I->Open(argv[I1]);
+        }
+    #else
+        for (int I1 = 1; I1 < ParamCount(); I1++)
+            I->Open(Ztring().From_Local(ParamStr(I1).c_str()));
+    #endif
 }
 
 //***************************************************************************
@@ -227,25 +296,6 @@ __fastcall TMainF::TMainF(TComponent* Owner)
 //---------------------------------------------------------------------------
 void __fastcall TMainF::GUI_Configure()
 {
-    //Hard coded
-    float DPIScale=static_cast<float>(GetSystemDpiForProcess(GetCurrentProcess()))/96;
-    float ScaledScreenWidth=Screen->Width/DPIScale;
-    float ScaledScreenHeight=Screen->Height/DPIScale;
-    Width=500;
-    Height=400;
-    if (ScaledScreenWidth>=1024)
-        Width=700;
-    if (ScaledScreenWidth>=1280)
-        Width=830;
-    if (ScaledScreenHeight>=768)
-        Height=500;
-    if (ScaledScreenHeight>=1024)
-        Height=600;
-    Width*=DPIScale;
-    Height*=DPIScale;
-    Left=(Screen->Width-Width)/2;
-    Top=(Screen->Height-Height)/2;
-
     //Load Configuration
     if (Prefs->Config_Load()==2) //Showing options if no config
     {
@@ -256,6 +306,34 @@ void __fastcall TMainF::GUI_Configure()
         delete PreferencesF;
         Prefs->Config_Load(); //Again...
         #endif //MEDIAINFOGUI_PREFS_NO
+
+        //Quick temporary fix for translations after an update
+        //----------------------------------------------------------------------------------------------
+        //Copies updated language files from install folder to appdata folder if appdata folder exists,
+        //overwriting existing files
+        //----------------------------------------------------------------------------------------------
+        namespace fs = std::filesystem;
+        fs::path appdata_path = std::getenv("APPDATA");
+        fs::path language_path = appdata_path / "MediaInfo\\Plugin\\Language";
+        ZenLib::Ztring BaseFolder;
+        BaseFolder=Application->ExeName.c_str();
+        fs::path language_source_path = BaseFolder.substr(0, BaseFolder.rfind(__T("\\"))).c_str();
+        language_source_path = language_source_path / "Plugin\\Language";
+        if (fs::exists(language_path) && fs::is_directory(language_path)) {
+            try {
+                for (const auto& entry : fs::directory_iterator(language_source_path)) {
+                    fs::path current = entry.path();
+                    fs::path target = language_path / current.filename();
+                    if (fs::is_regular_file(current)) {
+                        fs::copy_file(current, target, fs::copy_options::overwrite_existing);
+                    }
+                }
+                //ShowMessage("Success updating translations!");
+            } catch (const fs::filesystem_error& e) {
+                ShowMessage("Error updating translations in %APPDATA%!");
+            }
+        }
+        //----------------------------------------------------------------------------------------------
     }
 
     // Removed, no more needed
@@ -324,50 +402,60 @@ void __fastcall TMainF::GUI_Configure()
     //Translation
     Translate();
 
-    //Refresh global
-    FormResize(NULL);
+    //Configure theme
+    ConfigTheme();
 }
 
 //---------------------------------------------------------------------------
 void __fastcall TMainF::FormShow(TObject *Sender)
 {
-    //Configuration of MediaInfoLib
-    if (I == NULL)
-    {
-        I = new MediaInfoList;
+    //Set window size and position
+    OSVERSIONINFO osvi;
+    ZeroMemory(&osvi, sizeof(OSVERSIONINFO));
+    osvi.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
+    GetVersionEx(&osvi);
+    int DPI;
+    if (osvi.dwMajorVersion >= 10 && (osvi.dwMajorVersion > 10 || osvi.dwMinorVersion > 0 || osvi.dwBuildNumber >= 14939))
+        DPI=GetDpiForWindow(WindowHandle);
+    else
+        DPI=GetDeviceCaps(GetDC(NULL), LOGPIXELSX);
+    float DPIScale=static_cast<float>(DPI)/96;
+    float ScaledScreenWidth=Screen->Width/DPIScale;
+    float ScaledScreenHeight=Screen->Height/DPIScale;
+    Width=500;
+    Height=400;
+    if (ScaledScreenWidth>=1024)
+        Width=700;
+    if (ScaledScreenWidth>=1280)
+        Width=830;
+    if (ScaledScreenHeight>=768)
+        Height=500;
+    if (ScaledScreenHeight>=1024)
+        Height=600;
+    Width*=DPIScale;
+    Height*=DPIScale;
+    Left=(Screen->Width-Width)/2;
+    Top=(Screen->Height-Height)/2;
 
-        //Load GUI preferences
-        GUI_Configure();
-
-        //File(s) in command line
-        #ifdef UNICODE
-            if (IsWin9X())
-            {
-                for (int I1 = 1; I1 <= ParamCount(); I1++)
-                    I->Open(ParamStr(I1).c_str());
-            }
-            else
-            {
-                int argc;
-                MediaInfoNameSpace::Char** argv = CommandLineToArgvW(GetCommandLineW(), &argc);
-                for (int I1 = 1; I1 < argc; I1++)
-                    I->Open(argv[I1]);
-            }
-        #else
-            for (int I1 = 1; I1 < ParamCount(); I1++)
-                I->Open(Ztring().From_Local(ParamStr(I1).c_str()));
-        #endif
-
-        Refresh();
-    }
+    //Refresh global
+    FormResize(NULL);
+    Refresh();
 }
 
 //---------------------------------------------------------------------------
 void __fastcall TMainF::FormClose(TObject *Sender, TCloseAction &Action)
 {
-    if (Page->ActivePage==Page_Sheet) //TODO : find WHY there is a crash without this...
-        ChangePage(Page_Easy);
+    //Delete temp HTML file on close
+    if (FileName_Temp!=__T(""))
+        File::Delete(FileName_Temp);
 
+    //The form is closed and all allocated memory for the form is freed.
+    Action = caFree;
+}
+
+//---------------------------------------------------------------------------
+void __fastcall TMainF::FormDestroy(TObject *Sender)
+{
     delete Prefs; Prefs=NULL;
     delete I; I=NULL;
 }
@@ -375,6 +463,10 @@ void __fastcall TMainF::FormClose(TObject *Sender, TCloseAction &Action)
 //---------------------------------------------------------------------------
 void __fastcall TMainF::FormResize(TObject *Sender)
 {
+    //If triggered on application close
+    if(Prefs==NULL)
+        return;
+
     //Main View
     Page->Left  =(ToolBar->Visible?ToolBar->Width:0)-2;
     Page->Width =ClientWidth-Page->Left+2;
@@ -460,7 +552,7 @@ void __fastcall TMainF::FormResize(TObject *Sender)
                 {
                     Page_Sheet_X_Web[KindOfStream]->Top =Page_Sheet_X[KindOfStream]->Top+1;
                     Page_Sheet_X_Web[KindOfStream]->Left=Page_Sheet_X[KindOfStream]->Width;
-				}
+                }
                 if (!Page_Sheet_X[KindOfStream+1]) //reached the bottom
                 {
                     //Bottom
@@ -566,6 +658,10 @@ void __fastcall TMainF::Translate()
     M_Options->Caption=Prefs->Translate(__T("Options")).c_str();
     M_Options_ShowToolBar->Caption=Prefs->Translate(__T("Show toolbar")).c_str();
     M_Options_ShowMenu->Caption=Prefs->Translate(__T("Show menu")).c_str();
+    M_Options_Theme->Caption=Prefs->Translate(__T("ColorTheme")).c_str();
+    M_Options_Theme_System->Caption=Prefs->Translate(__T("ColorTheme_System")).c_str();
+    M_Options_Theme_Light->Caption=Prefs->Translate(__T("ColorTheme_Light")).c_str();
+    M_Options_Theme_Dark->Caption=Prefs->Translate(__T("ColorTheme_Dark")).c_str();
     M_Options_CloseAllAuto->Caption=Prefs->Translate(__T("Close all before open")).c_str();
     M_Options_Preferences->Caption=Prefs->Translate(__T("Preferences")).c_str();
     M_Debug->Caption=Prefs->Translate(__T("Debug")).c_str();
@@ -703,6 +799,9 @@ void __fastcall TMainF::Refresh(TTabSheet *Page)
     if (!Prefs->Config(__T("EnableFfmpeg")).empty())
         I->Option_Static(__T("Enable_Ffmpeg"), Prefs->Config(__T("EnableFfmpeg")));
 
+    if (!Prefs->Config(__T("DisplayCaptions")).empty())
+        I->Option(__T("File_DisplayCaptions"), Prefs->Config(__T("DisplayCaptions")));
+
     //Easy
          if (Page==Page_Easy)
     {
@@ -885,9 +984,6 @@ void __fastcall TMainF::Refresh(TTabSheet *Page)
         if (!Prefs->Config(__T("InformTimestamp")).empty())
             I->Option_Static(__T("Inform_Timestamp"), Prefs->Config(__T("InformTimestamp")));
 
-        //Set language again to ensure proper display in case it was changed to XML etc.
-        I->Option_Static(__T("Language"), Prefs->Details[Prefs_Language].Read());
-
         Page_Text_Text->Text=I->Inform().c_str();
 
         //Specific in case of no file
@@ -904,6 +1000,9 @@ void __fastcall TMainF::Refresh(TTabSheet *Page)
             //Creating file
             Ztring S1=I->Inform().c_str();
             File F;
+            if (TStyleManager::ActiveStyle == TStyleManager::Style[DARK_MODE_STYLE]) {
+                S1=InjectDarkModeHTMLStyle(I->Inform().c_str());
+            }
             if (FileName_Temp==__T(""))
             {
                 FileName_Temp=FileName::TempFileName_Create(__T("MI_"));
@@ -920,9 +1019,12 @@ void __fastcall TMainF::Refresh(TTabSheet *Page)
         {
             Ztring TempA; TempA=Prefs->Translate(__T("At least one file"));
             Ztring Temp;
-            Temp+=L"about:<html><body>";
+            Temp+=L"about:<html><head></head><body>";
             Temp+=TempA.To_Unicode();
             Temp+=L"</body></html>";
+            if (TStyleManager::ActiveStyle == TStyleManager::Style[DARK_MODE_STYLE]) {
+                Temp=InjectDarkModeHTMLStyle(Temp.c_str());
+            }
             Page_HTML_HTML->Navigate((MediaInfoNameSpace::Char*)Temp.c_str());
         }
     }
@@ -980,7 +1082,7 @@ void __fastcall TMainF::Refresh(TTabSheet *Page)
             I->Option_Static(__T("Inform"), Prefs->Details[Prefs_Custom].Read());
         Ztring S1=I->Inform();
         if (S1.empty())
-			S1=Prefs->Translate(__T("At least one file")).c_str();
+            S1=Prefs->Translate(__T("At least one file")).c_str();
 
         if (I->Option_Static(__T("Inform_Get"), __T("")) == __T("Graph_Svg"))
         {
@@ -1203,7 +1305,7 @@ void __fastcall TMainF::M_File_Open_FolderClick(TObject *Sender)
     if (!FolderOpenDialog1->Execute(Handle))
         return;
 
-    if (TDirectory::GetFiles(FolderOpenDialog1->FileName).Length != 0) {
+    if (FolderOpenDialog1->FileName.Length != 0) {
         // First we clear the list
         if (M_Options_CloseAllAuto->Checked)
             M_File_Close_AllClick(Sender);
@@ -1333,9 +1435,6 @@ void __fastcall TMainF::M_View_HTMLClick(TObject *Sender)
 //---------------------------------------------------------------------------
 void __fastcall TMainF::M_View_XMLClick(TObject *Sender)
 {
-    //Language
-    I->Option_Static(__T("Language"), __T("raw"));
-
     M_View_XML->Checked=true;
     ToolBar_View_XML->Checked=true;
     ChangePage(Page_Custom);
@@ -1344,9 +1443,6 @@ void __fastcall TMainF::M_View_XMLClick(TObject *Sender)
 //---------------------------------------------------------------------------
 void __fastcall TMainF::M_View_JSONClick(TObject *Sender)
 {
-    //Language
-    I->Option_Static(__T("Language"), __T("raw"));
-
     M_View_JSON->Checked=true;
     ToolBar_View_JSON->Checked=true;
     ChangePage(Page_Custom);
@@ -1852,9 +1948,6 @@ void __fastcall TMainF::Page_Sheet_Change(TObject *Sender)
 
     I->Option(__T("Inform"), __T(""));
 
-    //Set language again to ensure proper display in case it was changed to XML etc.
-    I->Option_Static(__T("Language"), Prefs->Details[Prefs_Language].Read());
-
     Page_Sheet_Text->Text=I->Get(Page_Position, (stream_t)KindOfStream, I1, __T("Inform")).c_str();
 }
 
@@ -2048,15 +2141,27 @@ void __fastcall TMainF::Footer_ButtonClick(TObject *Sender)
 }
 
 //---------------------------------------------------------------------------
-void __fastcall TMainF::M_Options_DarkmodeClick(TObject* Sender)
+void __fastcall TMainF::M_Options_Theme_SystemClick(TObject *Sender)
 {
-    if (M_Options_Darkmode->Checked) {
-        TStyleManager::TrySetStyle(LIGHT_MODE_STYLE, false);
-        M_Options_Darkmode->Checked = false;
-    } else {
-        TStyleManager::TrySetStyle(DARK_MODE_STYLE, false);
-        M_Options_Darkmode->Checked = true;
-    }
+    Prefs->Config(__T("Theme")) = __T("0");
+    Prefs->Config.Save();
+    ConfigTheme();
+}
+
+//---------------------------------------------------------------------------
+void __fastcall TMainF::M_Options_Theme_LightClick(TObject *Sender)
+{
+    Prefs->Config(__T("Theme")) = __T("1");
+    Prefs->Config.Save();
+    ConfigTheme();
+}
+
+//---------------------------------------------------------------------------
+void __fastcall TMainF::M_Options_Theme_DarkClick(TObject *Sender)
+{
+    Prefs->Config(__T("Theme")) = __T("2");
+    Prefs->Config.Save();
+    ConfigTheme();
 }
 
 //---------------------------------------------------------------------------
@@ -2064,13 +2169,7 @@ void __fastcall TMainF::ApplicationEvents1OnSettingChange(
     TObject* Sender, int Flag, const UnicodeString Section, int &Result)
 {
     if (Section == "ImmersiveColorSet") {
-        if (WindowsDarkModeEnabled()) {
-            TStyleManager::TrySetStyle(DARK_MODE_STYLE, false);
-            M_Options_Darkmode->Checked = true;
-        } else {
-            TStyleManager::TrySetStyle(LIGHT_MODE_STYLE, false);
-            M_Options_Darkmode->Checked = false;
-        }
+        ConfigTheme();
     }
 }
 
